@@ -61,9 +61,9 @@ namespace ojph {
                                ui32 comp_num, ui32 res_num)
     {
       mem_fixed_allocator* allocator = codestream->get_allocator();
-      const param_cod* cdp = codestream->get_cod(comp_num);
-      ui32 t = cdp->get_num_decompositions()
-             - codestream->get_skipped_res_for_recon();
+      const param_cod* cdp = codestream->get_coc(comp_num);
+      ui32 num_decomps = cdp->get_num_decompositions();
+      ui32 t = num_decomps - codestream->get_skipped_res_for_recon();
       bool skipped_res_for_recon = res_num > t;
 
       const param_atk* atk = cdp->access_atk();
@@ -85,7 +85,6 @@ namespace ojph {
               "with index %d, but there are no such marker within the "
               "main codestream headers", dfs_idx);
           }
-          ui32 num_decomps = cdp->get_num_decompositions();
           ds = dfs->get_dwt_type(num_decomps - res_num + 1);
         }
       }
@@ -199,9 +198,6 @@ namespace ojph {
         allocator->pre_alloc_obj<precinct>((size_t)num_precincts.area());
       }
 
-      const param_siz* szp = codestream->get_siz();
-      ui32 precision = cdp->propose_precision(szp, comp_num);
-
       //allocate lines
       if (skipped_res_for_recon == false)
       {
@@ -209,19 +205,33 @@ namespace ojph {
         allocator->pre_alloc_obj<line_buf>(num_steps + 2);
         allocator->pre_alloc_obj<lifting_buf>(num_steps + 2);
 
+        const param_qcd* qp = codestream->access_qcd()->get_qcc(comp_num);
+        ui32 precision = qp->propose_precision(cdp);
+        const param_atk* atk = cdp->access_atk();
+        bool reversible = atk->is_reversible();
+
         ui32 width = res_rect.siz.w + 1;
-        if (precision <= 32) {
-          for (ui32 i = 0; i < num_steps; ++i)
-            allocator->pre_alloc_data<si32>(width, 1);
-          allocator->pre_alloc_data<si32>(width, 1);
-          allocator->pre_alloc_data<si32>(width, 1);
-        }
-        else 
+        if (reversible)
         {
-          for (ui32 i = 0; i < num_steps; ++i)
+          if (precision <= 32) {
+            for (ui32 i = 0; i < num_steps; ++i)
+              allocator->pre_alloc_data<si32>(width, 1);
+            allocator->pre_alloc_data<si32>(width, 1);
+            allocator->pre_alloc_data<si32>(width, 1);
+          }
+          else 
+          {
+            for (ui32 i = 0; i < num_steps; ++i)
+              allocator->pre_alloc_data<si64>(width, 1);
             allocator->pre_alloc_data<si64>(width, 1);
-          allocator->pre_alloc_data<si64>(width, 1);
-          allocator->pre_alloc_data<si64>(width, 1);
+            allocator->pre_alloc_data<si64>(width, 1);
+          }
+        }
+        else {
+          for (ui32 i = 0; i < num_steps; ++i)
+            allocator->pre_alloc_data<float>(width, 1);
+          allocator->pre_alloc_data<float>(width, 1);
+          allocator->pre_alloc_data<float>(width, 1);
         }
       }
     }
@@ -237,7 +247,7 @@ namespace ojph {
     {
       mem_fixed_allocator* allocator = codestream->get_allocator();
       elastic = codestream->get_elastic_alloc();
-      const param_cod* cdp = codestream->get_cod(comp_num);
+      const param_cod* cdp = codestream->get_coc(comp_num);
       ui32 t, num_decomps = cdp->get_num_decompositions();
       t = num_decomps - codestream->get_skipped_res_for_recon();
       skipped_res_for_recon = res_num > t;
@@ -448,9 +458,6 @@ namespace ojph {
         level_index[i] = level_index[i - 1] + val;
       cur_precinct_loc = point(0, 0);
 
-      const param_siz* szp = codestream->get_siz();
-      ui32 precision = cdp->propose_precision(szp, comp_num);
-
       //allocate lines
       if (skipped_res_for_recon == false)
       {
@@ -473,23 +480,43 @@ namespace ojph {
         new (aug) lifting_buf;
         aug->line = lines + num_steps + 1;
 
+        const param_qcd* qp = codestream->access_qcd()->get_qcc(comp_num);
+        ui32 precision = qp->propose_precision(cdp);
+
         // initiate storage of line_buf
         ui32 width = res_rect.siz.w + 1;
-        if (precision <= 32)
+        if (this->reversible)
         {
-          for (ui32 i = 0; i < num_steps; ++i)
-            ssp[i].line->wrap(
+          if (precision <= 32)
+          {
+            for (ui32 i = 0; i < num_steps; ++i)
+              ssp[i].line->wrap(
+                allocator->post_alloc_data<si32>(width, 1), width, 1);
+            sig->line->wrap(
               allocator->post_alloc_data<si32>(width, 1), width, 1);
-          sig->line->wrap(allocator->post_alloc_data<si32>(width, 1), width, 1);
-          aug->line->wrap(allocator->post_alloc_data<si32>(width, 1), width, 1);
-        }
-        else
-        {
-          for (ui32 i = 0; i < num_steps; ++i)
-            ssp[i].line->wrap(
+            aug->line->wrap(
+              allocator->post_alloc_data<si32>(width, 1), width, 1);
+          }
+          else
+          {
+            for (ui32 i = 0; i < num_steps; ++i)
+              ssp[i].line->wrap(
+                allocator->post_alloc_data<si64>(width, 1), width, 1);
+            sig->line->wrap(
               allocator->post_alloc_data<si64>(width, 1), width, 1);
-          sig->line->wrap(allocator->post_alloc_data<si64>(width, 1), width, 1);
-          aug->line->wrap(allocator->post_alloc_data<si64>(width, 1), width, 1);
+            aug->line->wrap(
+              allocator->post_alloc_data<si64>(width, 1), width, 1);
+          }
+        }
+        else 
+        {
+            for (ui32 i = 0; i < num_steps; ++i)
+              ssp[i].line->wrap(
+                allocator->post_alloc_data<float>(width, 1), width, 1);
+            sig->line->wrap(
+              allocator->post_alloc_data<float>(width, 1), width, 1);
+            aug->line->wrap(
+              allocator->post_alloc_data<float>(width, 1), width, 1);
         }
 
         cur_line = 0;
@@ -586,9 +613,19 @@ namespace ojph {
           else
           {
             // vertical transform
-            si32* sp = aug->line->i32;
-            for (ui32 i = width; i > 0; --i)
-              *sp++ <<= 1;
+            if (aug->line->flags & line_buf::LFT_32BIT)
+            {
+              si32* sp = aug->line->i32;
+              for (ui32 i = width; i > 0; --i)
+                *sp++ <<= 1;
+            }
+            else
+            {
+              assert(aug->line->flags & line_buf::LFT_64BIT);
+              si64* sp = aug->line->i64;
+              for (ui32 i = width; i > 0; --i)
+                *sp++ <<= 1;
+            }
             // horizontal transform
             rev_horz_ana(atk, bands[2].get_line(),
               bands[3].get_line(), aug->line, width, horz_even);
@@ -1002,13 +1039,11 @@ namespace ojph {
     {
       if (this->res_num == resolution_num)
         return get_num_bytes();
-      else {
-        if (child_res)
-          return child_res->get_num_bytes(resolution_num);
-        else
-          return 0;
+      if (resolution_num < this->res_num) {
+        assert(child_res);
+        return child_res->get_num_bytes(resolution_num);
       }
-
+      return 0;
     }
   }
 }
\ No newline at end of file
